<?php
namespace Teknisa\CMP\Modules\NecessidadeCompras\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Setforms2\Shared\ORM\DateTime;
use Setforms2\Shared\ORM\DateInterval;
use Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class NecessidadeCompras  {

    protected $environment;
	protected $dataSourceManager;
	protected $connection;
	protected $entityManager;
    
    public function __construct(Environment $environment, $dataSourceManager, $connection, $entityManager){
		$this->environment = $environment;
		$this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
        $this->entityManager = $entityManager;
	}
    
    public function parasupriNececomp($cdfilial){
        $params = array(
    		"P_CDFILIAL" => $cdfilial
    	);
    	
    	$sql = "SELECT CDFILIAL, IDJUSSOLEXT, IDBLOQALTNEC, IDAUDALTSOL, IDULTIDTULTI, IDULTCOMPSOL, IDULTDESTSOL, CDGRUPOPER , IDULTIDTVALQ, IDVALDTAF , IDBLOQUEAAF , IDMARSEMQUA, IDTIPOAPUR, IDPROSEMMAR, IDPROSEMFOR , IDUTQTDEVOLU, IDBAIXSOLDFS, IDCONSPRSOL, IDEXJUSTESCM , IDBLOQALTSOLAUT FROM PARASUPRI WHERE CDFILIAL = :P_CDFILIAL";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	
    	return $query->fetchAll();
    }
    
    public function selectCentral(){
        $params = array(
    		"P_CDFILIAL" => $this->environment->getCdfilial(),
    		"NRORG" => $this->environment->getNrorg()
    	);
    	
    	$sql = "SELECT NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS CENTRALCMP, FIL.NMFILIAL AS NMFILIAL_CENTRAL FROM FILIAL FIL, FILIAL WHERE FILIAL.CDFILIAL = :P_CDFILIAL AND NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = FIL.CDFILIAL AND FILIAL.NRORG = FIL.NRORG AND FILIAL.NRORG = :NRORG";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$this->central = $query->fetchAll();
    }
    
    public function retornaParasupriNececomp(DTO\Request\Row $request, DTO\Response $response){
        try {
            $this->selectCentral();
            $parasupri = $this->parasupriNececomp($this->central[0]['CENTRALCMP']);
            
            $response->addDataSet(new DataSet("parasupri", $parasupri));
        }
        catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function GERA_SOLICITACAO($nececomp, $extra, $msg) {
        $connection = $this->connection;
        
        $procedure = new StoredProcedure($connection, 'GERA_SOLICITACAO');
        $procedure->addParam(new Param('P_CDFILILANC', Param::PARAM_INPUT, $this->environment->getCdfilial(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRNECECOMP', Param::PARAM_INPUT, $nececomp, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERLANC', Param::PARAM_INPUT, $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_EXTRA', Param::PARAM_INPUT, $extra, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_MSGEXTRA', Param::PARAM_INPUT, $msg, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_MOBILE', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function selectSolicita($nececomp){
        $params = array(
    		"P_NRNECECMPSOL" => $nececomp
    	);

    	$sql = "SELECT MAX(NRSOLICMP) NRSOLICMP FROM SOLICITA WHERE NRNECECMPSOL = :P_NRNECECMPSOL";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$this->solicita = $query->fetchAll();

    }
    
    public function GetPrxSoliComp(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();

        try {
            $this->connection->beginTransaction();
            
            if(array_key_exists('PRODUTOS', $row)) {
                self::salvaProdutos($row);
            }
            
            $this->GERA_SOLICITACAO($row['NRNECECOMP'], $row['EXTRA'], $row['MENSAGEM']);
            $this->selectSolicita($row['NRNECECOMP']);
            $resposta = $this->solicita[0]['NRSOLICMP'];

            $response->addNotification(new DTO\Response\Notification("NRSOLICMP", $resposta));
            
            $this->connection->commit();
        }
        catch(\Exception $e){
            $this->connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteNececomp(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        
        $params = array(
    		"P_NRNECECOMP" => $row[0]
    	);

    	$sql = "DELETE FROM TRABNECECOMP WHERE NRNECECOMP = :P_NRNECECOMP";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
       
    }
    
    public function checkProdAtivo(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        
        $params = array(
    		"P_CDPRODUTO" => $row['CDPRODUTO'],
    		"P_CDFILIAL" => $row['CDFILIAL']
    	);

    	$sql = "SELECT PRODFILI.IDPRODATIV, PRODUTO.IDPRODATIVPR FROM PRODFILI, PRODUTO WHERE PRODFILI.CDFILIAL = :P_CDFILIAL AND PRODUTO.CDPRODUTO = :P_CDPRODUTO AND PRODFILI.CDPRODUTO = PRODUTO.CDPRODUTO";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$resposta = $query->fetchAll();

        $response->addDataSet(new DataSet("resposta", $resposta));
        
    }
    
    public function insertProdNeceComp($produto){
        
        $params = array(
    		"CDFILNECECMP" => $produto['CDFILIAL'],
    		"NRNECECOMP"   => $produto['NRNECECOMP'],
    		"CDPRODNECCMP" => $produto['CDPRODUTO'],
    		"DTENTRNECCMP" => $produto['DTENTRNECCMP'],
    		"IDMARCNECCMP" => 'S',
    		"DTPRIUTIPROD" => $produto['DTPRIUTIPROD'],
    		"QTPRCNVCOMPR" => $produto['QTPRCNVCOMPR'],
    		"QTPRACOMPRAR" => $produto['QTPRCNVCOMPR'],
    		"NRORG" => $this->environment->getNrorg(),
    	);

    	$sql = "INSERT INTO TRABNECECOMP ( CDFILNECECMP, NRNECECOMP, DTENTRNECCMP, CDPRODNECCMP, IDMARCNECCMP, DTPRIUTIPROD, QTPRCNVCOMPR, QTPRACOMPRAR, NRORG) VALUES ( :CDFILNECECMP, :NRNECECOMP, TO_DATE(:DTENTRNECCMP, 'DD/MM/YYYY'), :CDPRODNECCMP, :IDMARCNECCMP, TO_DATE(:DTPRIUTIPROD, 'DD/MM/YYYY'), :QTPRCNVCOMPR, :QTPRACOMPRAR, :NRORG)";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
        
    }
    
    public function updateItemDesmarcado($produto) {
        $params = array(
    		"P_NRNECECOMP"      => $produto['NRNECECOMP'],
    		"P_DTENTRNECCMP"    => $produto['DTENTRNECCMP'],
    		"P_CDPRODNECCMP"    => $produto['CDPRODUTO'],
    		"P_CDFILNECECMP"    => $produto['CDFILNECECMP'],
    	);

    	$sql = "UPDATE TRABNECECOMP SET IDMARCNECCMP = 'N' WHERE NRNECECOMP = :P_NRNECECOMP AND DTENTRNECCMP = TO_DATE(:P_DTENTRNECCMP, 'DD/MM/YYYY') AND CDPRODNECCMP = :P_CDPRODNECCMP AND CDFILNECECMP = :P_CDFILNECECMP";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    
    public function updateQuantidadeItem($produto) {
        $params = array(
    		"P_NRNECECOMP"      => $produto['NRNECECOMP'],
    		"P_QTPRCNVCOMPR"    => $produto['QTPRCNVCOMPR'],
    		"P_DTENTRNECCMP"    => $produto['DTENTRNECCMP'],
    		"P_CDPRODNECCMP"    => $produto['CDPRODUTO'],
    		"P_QTPRACOMPRAR"    => $produto['QTPRCNVCOMPR'],
    		"P_CDFILNECECMP"    => $produto['CDFILNECECMP'],
    	);

    	$sql = "UPDATE TRABNECECOMP SET QTPRCNVCOMPR = :P_QTPRCNVCOMPR, QTPRACOMPRAR = :P_QTPRACOMPRAR WHERE NRNECECOMP = :P_NRNECECOMP AND DTENTRNECCMP = TO_DATE(:P_DTENTRNECCMP, 'DD/MM/YYYY') AND CDPRODNECCMP = :P_CDPRODNECCMP  AND CDFILNECECMP = :P_CDFILNECECMP";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    
    public function salvaProdutos($row) {
        $produtos = $row['PRODUTOS'];
        
        foreach ($produtos as $produto) {
            $produto['CDFILNECECMP'] = $row['CDFILNECECMP'];
            $produto['DTENTRNECCMP'] = self::parseDate($produto['DTENTRNECCMP']);
            
            if ($produto['__isSelected']) {
                if ($produto['__is_new']) {
                    self::insertProdNeceComp($produto);
                } else if (array_key_exists('ALTERADO', $produto) && $produto['ALTERADO']) {
                    self::updateQuantidadeItem($produto);
                }
            } else {
                self::updateItemDesmarcado($produto);
            }
        }
    }
    
    private function parseDate($str) {
        preg_match("/\d{1,2}\/\d{1,2}\/\d{2,4}/", $str, $matches);
        
        return $matches[0];
    }
}
